import os
from datetime import datetime
from publishing_boy import config

"""Here are functions that serve the
role of plugins. Each function
accepts tuple object with:
- filename
- content
- path

Those functions are loaded to the program
by a decorator, during module loading.
Functions are stored inside a list.
"""

PLUGINS = []


def register_plugin(fn):
    """This decorator add a function to PLUGINS
    list.

    Order of plugins is not important"""
    PLUGINS.append(fn)

    return fn


@register_plugin
def content_function(obj):
    """Just return content"""
    _, _, _, content = obj

    return 'content', content


@register_plugin
def title_extractior(obj):
    """Extract title from content.
    Use NTLK to do stuff with text. - maybe later

    for know i will use first sentence in text

    @return: 'title', generated_content"""
    _, _, _, content = obj

    if not content:
        return 'title', ''

    cut  = 40 if len(content) >= 40 else len(content)

    title = content[:cut].strip() + " ..."
    pos = content.find(".")

    if pos != -1:
        title = content[:pos].strip()

    return 'title', title


@register_plugin
def creation_date(obj):
    """Extract date when the file was
    created.

    @return: 'date', date(YYYY-mm-dd HH:MM:SS)"""
    _, _, abspath, _ = obj
    return 'cdate', datetime.fromtimestamp(os.path.getctime(abspath))


@register_plugin
def modified_date(obj):
    """Extract date when the file was
    modified.

    @return: 'modified', date(YYYY-mm-dd HH:MM:SS)"""
    _, _, abspath, _ = obj
    return 'mdate', datetime.fromtimestamp(os.path.getmtime(abspath))


@register_plugin
def category_extract(obj):
    """Extract category. Category are
    the folder names in path file.

    @return: 'category', 'String, Separated, by'
    """
    _, filepath, _, _ = obj
    names = filter(None, os.path.dirname(filepath).split("/"))
    categories = ", ".join(map(str.capitalize, names))

    return 'categories', categories


@register_plugin
def authors(obj):
    """Return authors"""
    return 'authors', config['Posts'].get('author', '')


def run_plugins(obj):
    """Run all plugins on object"""
    return {key: value for key, value in (plugin(obj) for plugin in PLUGINS)}



import os
from django.core.files.base import ContentFile
from publishing_boy.tests.utils import get_test_storage

temp_dir, storage = get_test_storage()

filename, path = 'test_file.md', 'test_file.md'
content = """
You have to do it Nicky. In order to do that, it’s important to have both overlapping and complementary skills on your team: A good rule of thumb is that any task should have at least two people who can do it, and any two people should have a number of significant tasks where one would obviously be better suited to work on it than another. The former is much more important than the latter, but both are important."""

storage.save(filename, ContentFile(content))

obj1 = (filename,
       filename,
       os.path.abspath(storage.path(filename)),
       content,)



# -*- coding: utf-8 -*-
from publishing_boy.template import render
from publishing_boy.process import (
    create_content_folder,
    file_tuples,
    save_content,
)

from publishing_boy.plugins import run_plugins
"""Main module."""


def transform(obj):
    """This function has to take the object
    and then transform it using plugin functions.

    the output is the object with replaced content
    that suits pelican static site generator.
    """
    name , filepath, fullpath, _ = obj
    context = run_plugins(obj)

    return name, filepath, fullpath, render(context)


def process(folder):
    """Process files in a given folder.

    Create data tuples from files and their contents.
    Pass each tuple thorough transformation function,
    and save it into the  content/ folder.

    """
    for obj in file_tuples(folder):
        save_content(transform(obj))



import shutil
from datetime import datetime, timedelta
from publishing_boy import config
from django.core.files.base import ContentFile

from publishing_boy.plugins import (
    title_extractior,
    creation_date,
    modified_date,
    category_extract,
    authors,
)


from tests.fixtures import (
    temp_dir,
    storage,
    filename,
    path,
    content,
    obj1 as obj,
)

content = """i want to use pelican publishing to generate content.

no styles - simply some non-white background and
some almost-black text - it has to be good for my
eyes. probably some sepia background and some graphite text.

fuck the titles, i will use markdown to write
stuff. i will write stuff, then i create a pipleine
in jenkins that will generate title using ntlk,
simply some main words, i will refine this and 
use some ai or shit to generate titles
but for now i will have tags
also this will be put into a template of pelican content
so this will be transformation of one set of files
into another set of files and this will transform
into html files

that will be send later to pelican and that will generate
content, this will publish stuff to github, and
from github, hetzner will download master branch
each 6 hours.

i also started a .command file which will store my commands
for greater future good

my programming style turns out to be more data flow style,
where the data are being transformed into desired output."""


def teardown_module():
    shutil.rmtree(temp_dir)


def test_title_extractor():
    """Test title_extractor"""
    f2 = (
        '',
        'file.c',
        '',
        'A b c d',
    )
    f3 = (
        '',
        'a/b/c/file.c',
        '',
        '',
    )
    f4 = (
        '',
        'a/b/c/file.c',
        '',
        content,
    )

    label, r1 = title_extractior(obj)
    _, r2 = title_extractior(f2)
    _, r3 = title_extractior(f3)
    _, r4 = title_extractior(f4)

    assert label == 'title'
    assert r1 == 'You have to do it Nicky'
    assert r2 == 'A b c d ...'
    assert r3 == ''
    assert r4 == 'i want to use pelican publishing to generate content'


def test_creation_date():
    """Test for creation_date"""
    ctime = storage.get_created_time(filename)
    label, result_ctime = creation_date(obj)

    assert label == 'cdate'
    assert ctime == result_ctime
    assert datetime.now() - result_ctime < timedelta(seconds=2)


def test_modified_date():
    """Test getting the modification date"""
    storage.save('test.file', ContentFile('custom contents'))

    mtime = storage.get_modified_time(filename)
    label, result_mtime = modified_date(obj)

    assert label == 'mdate'
    assert mtime == result_mtime

    assert datetime.now() - result_mtime < timedelta(seconds=2)


def test_category_extract():
    """Test category extract"""

    f1 = (
        '',
        'file.c',
        '',
        '',
    )
    f2 = (
        '',
        'a/b/c/file.c',
        '',
        '',
    )
    f3 = (
        '',
        '/file.c',
        '',
        '',
    )

    label, r1 = category_extract(f1)
    _, r2 = category_extract(f2)
    _, r3 = category_extract(f3)

    assert label == 'categories'
    assert r1 == ''
    assert r2 == 'A, B, C'
    assert r3 == ''


def test_authors():
    def _author():
        return config['Posts'].get('author', '')

    label, author = authors(obj)

    assert label == 'authors'
    assert author == _author()



from django.core.files.base import File, ContentFile
import os
import shutil
import unittest
from _pytest.monkeypatch import MonkeyPatch
from publishing_boy import config
from publishing_boy.tests.utils import get_test_storage
from publishing_boy.process import build_tuple, file_tuples, save_content, create_content_folder


class ProcessTest(unittest.TestCase):

    def setUp(self):
        """Prepare file tests"""
        self.monkeypatch = MonkeyPatch()
        self.filename = 'testing/test_post.md'
        self.content = 'This is my content.'
        temp_dir, storage = get_test_storage()
        self.temp_dir = temp_dir
        self.storage = storage
        self.storage.save(self.filename, ContentFile(self.content))

        self.content_temp_dir, self.content_storage = get_test_storage()

    def result_tuple(self):
        return (os.path.basename(self.filename),
                self.filename,
                os.path.abspath(self.storage.path(self.filename)),
                self.content,)

    def tearDown(self):
        """Remove temp dir"""
        shutil.rmtree(self.temp_dir)
        shutil.rmtree(self.content_temp_dir)

    def test_setup(self):
        self.assertFalse(self.storage.exists('testing/second.md'))
        self.assertTrue(self.storage.exists(self.filename))

    def test_build_tuple(self):
        _, path, abspath, _ = self.result_tuple()
        result = build_tuple(path, abspath)

        assert result == self.result_tuple()

    def test_file_tuples(self):
        assert list(file_tuples(self.storage.path(''))) == [self.result_tuple()]

    def test_save_content(self):

        config['Instance'] = {'content_dir': self.content_temp_dir}

        saved_filename = save_content(self.result_tuple())

        assert saved_filename
        assert self.content_storage.exists(saved_filename)

    def test_create_content_folder(self):
        folders = [os.path.join(self.temp_dir, test_dir) for test_dir
                   in ['a', 'b/c', 'd/e/f']]

        for folder in folders:
            with self.subTest():
                create_content_folder(folder)
                assert self.storage.exists(folder)

        # try doing the same thing one more time
        for folder in folders:
            with self.subTest():
                create_content_folder(folder)
                assert self.storage.exists(folder)



# -*- coding: utf-8 -*-
from publishing_boy.publishing_boy import process
from publishing_boy.process import create_content_folder

"""Console script for publishing_boy."""
import sys
import click


@click.command()
@click.argument('input_folder')
@click.argument('output_folder')
def main(input_folder, output_folder):
    """Console script for publishing_boy."""
    click.echo('Running publishing boy...')
    create_content_folder(output_folder)

    report = process(input_folder)

    return 0


if __name__ == "__main__":
    sys.exit(main())  # pragma: no cover



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `publishing_boy` package."""
import os
import pytest

from click.testing import CliRunner
from publishing_boy.cli import main


TEST_FILE = 'test.md'
TEST_FOLDER = 'test_folder'
TEST_OUTPUT_FILE = 'test_output/test.md'


def test_command_line_folders():
    """Go to cookiecutter download folder, and
    extract the contents with cli testing.

    Test different types of input folders
    """
    from tests.fixtures import ContentFile, get_test_storage, filename, content

    temp_dir, storage = get_test_storage()

    input_folder = os.path.join(temp_dir, 'input')
    output_folder = os.path.join(temp_dir, 'output')

    os.makedirs(input_folder)

    filepath = os.path.join(input_folder, filename)
    result_filename = os.path.join(output_folder, filename)

    storage.save(filepath, ContentFile(content))

    assert storage.exists(result_filename) == False

    runner = CliRunner()
    result = runner.invoke(main, [input_folder, output_folder])

    assert result.exit_code == 0
    assert storage.exists(output_folder)
    assert storage.exists(result_filename)



# -*- coding: utf-8 -*-
import os
import configparser
"""Top-level package for Publishing Boy."""

__author__ = """Red"""
__email__ = 'przemyslaw.kot@gmail.com'
__version__ = '0.1.0'

PATH = os.path.dirname(os.path.abspath(__file__))

config_file = os.path.join(PATH, 'settings.cfg')

config = configparser.ConfigParser()
config.read(config_file)



import glob
import os
from django.core.files.base import File, ContentFile
from publishing_boy import config
from publishing_boy.tests.utils import get_storage


def build_tuple(filepath, abspath):
    assert os.path.exists(abspath)

    filename = os.path.basename(filepath)

    content = ''

    with open(abspath, 'r') as f:
        content = f.read()

    return filename, filepath, abspath, content


def file_tuples(folder):
    """Generate file tuples from files in folder"""
    query = os.path.join(folder, '**/*.md')
    results = glob.glob(query, recursive=True)
    for abspath in results:
        filepath = str.lstrip(abspath.split(folder)[1], '/')  # substract folder
        yield build_tuple(filepath, abspath)


def save_content(obj):
    """Save content as a flat file """
    filename, _, _, content = obj
    storage = get_storage(config['Instance']['content_dir'])
    return storage.save(filename, ContentFile(content))


def create_content_folder(output_path):
    """Create in a folder current to running the software
    folder named 'content' that will keep the output."""
    config['Instance'] = {}
    config['Instance']['content_dir'] = output_path

    try:
        os.makedirs(output_path)
    except FileExistsError:
        pass



import datetime
from publishing_boy.template import render


TEMPLATE = """
Title: A b c
Date: 2019-01-01 01:00:00
Modified: 2019-01-01 01:00:00
Category: A, B, C
Authors: A. B

A b c. E f g. H i j.
"""

date_format = '%Y-%m-%d %H:%M:%S'

context = {
    'title': 'A b c',
    'cdate': datetime.datetime.strptime('2019-01-01 01:00:00', date_format),
    'mdate': datetime.datetime.strptime('2019-01-01 01:00:00', date_format),
    'categories': 'A, B, C',
    'authors': 'A. B',
    'content': 'A b c. E f g. H i j.',
}


def test_render():
    assert TEMPLATE == render(context)



import os
from publishing_boy.publishing_boy import transform
from tests.fixtures import ContentFile, get_test_storage, filename, content
import shutil


def test_transform():
    global filename, content
    temp_dir, storage = get_test_storage()

    storage.save(filename, ContentFile(content))

    obj = (filename,
       filename,
       os.path.abspath(storage.path(filename)),
       content,)

    from publishing_boy.plugins import (
        authors,
        title_extractior,
        creation_date,
        modified_date,
        category_extract,
        content_function,
    )

    result = transform(obj)

    filename, filepath, abspath, content = obj
    r_f, r_fp, r_abs, f_content = result
    assert r_f == filename
    assert r_fp == filepath
    assert r_abs == abspath

    assert f_content.find('Title:') > 0
    assert f_content.find('Date:') > 0
    assert f_content.find('Modified:') > 0
    assert f_content.find('Category:') > 0
    assert f_content.find('Authors:') > 0

    shutil.rmtree(temp_dir)



TEMPLATE = """title: {title}
date: {cdate}
modified: {mdate}
category: {categories}
authors: {authors}

{content}
"""


def render(context):
    """Use simple template render to generate content"""
    return TEMPLATE.format(**context)
